// Implementation of the Least Significant Half of the MISR Layer
module misr_8bit_second_half (
  input [7:0]d,
  input [7:0]y,
  input enable,clock,reset,
  output [7:0]q
  );
  
  wire [7:0]out; // the outputs of the CUTs Layer
  wire [7:0]o; // the outputs of the multiplexers
  //wire [7:0] y; // the outputs of the BILBO (Watchdog) Unit
  
  mux_1s_1b mux0 ( .d0(q[0] ^ out[0]),
                   .d1(q[0]),
                   .s(y[0]),
                   .o(o[0])
                  );
                  
  mux_1s_1b mux1 ( .d0(q[1] ^ out[1]),
                   .d1(q[1]),
                   .s(y[1]),
                   .o(o[1])
                  );
                  
  mux_1s_1b mux2 ( .d0(q[2] ^ out[2]),
                   .d1(q[2]),
                   .s(y[2]),
                   .o(o[2])
                  );
  
  mux_1s_1b mux3 ( .d0(q[3] ^ out[3]),
                   .d1(q[2]),
                   .s(y[3]),
                   .o(o[3])
                  );
                  
  mux_1s_1b mux4 ( .d0(q[4] ^ out[4]),
                   .d1(q[3]),
                   .s(y[4]),
                   .o(o[3])
                  );
                  
  mux_1s_1b mux5 ( .d0(q[5] ^ out[5]),
                   .d1(q[5]),
                   .s(y[5]),
                   .o(o[5])
                  );
                  
  mux_1s_1b mux6 ( .d0(q[6] ^ out[6]),
                   .d1(q[6]),
                   .s(y[6]),
                   .o(o[6])
                  );
                  
  mux_1s_1b mux7 ( .d0(q[7] ^ out[7]),
                   .d1(q[7]),
                   .s(y[7]),
                   .o(o[7])
                  );
  
  dff_ar_rec DFF0 ( .d(q[7] ^ q[0]),
                    .enable(enable),
                    .reset(reset),
                    .clock(clock),
                    .q(q[0])
                   );
                   
  dff_ar_rec DFF1 ( .d(o[0]),
                    .enable(enable),
                    .reset(reset),
                    .clock(clock),
                    .q(q[1])
                   );
  
  dff_ar_rec DFF2 ( .d(o[1]),
                    .enable(enable),
                    .reset(reset),
                    .clock(clock),
                    .q(q[2])
                   );
                   
  dff_ar_rec DFF3 ( .d(o[2]),
                    .enable(enable),
                    .reset(reset),
                    .clock(clock),
                    .q(q[3])
                   );
                   
  dff_ar_rec DFF4 ( .d(o[3]),
                    .enable(enable),
                    .reset(reset),
                    .clock(clock),
                    .q(q[4])
                   );

  dff_ar_rec DFF5 ( .d(o[4]),
                    .enable(enable),
                    .reset(reset),
                    .clock(clock),
                    .q(q[5])
                   );
                   
  dff_ar_rec DFF6 ( .d(o[5]),
                    .enable(enable),
                    .reset(reset),
                    .clock(clock),
                    .q(q[6])
                   );
                   
                   
  dff_ar_rec DFF7 ( .d(o[6]),
                    .enable(enable),
                    .reset(reset),
                    .clock(clock),
                    .q(q[7])
                   );
                   
endmodule

