// Instantiation of the entire Reconfigurable Built-In Self-Test Architecture
module reconfigurable_bist (
  input [3:0] PV, 
  input clock, reset,
  output [15:0]q
  );
  
  wire ISD_o; // output of the Idle State Detector
  wire [15:0]In; // outputs of the LFSR Layer
  wire [15:0] i; // outputs of the first batch of Tri-State gates
  wire [15:0] Out; // outputs of the CUTs Layer
  wire [15:0] ROMOut; // outputs of the CUTs Layer
  wire [15:0] o; // outputs of the second batch of Tri-State gates
  wire [7:0] w; // outputs of the watchdog goes to second half of misr
  wire weee;
  wire [15:0]countt;
  reg [15:0]count;
  reg wee;
  
  always @(posedge clock)begin
	count <= count + 16'b1;
	if (count <65535)
	begin 
	wee <= 1; 
end
end
 assign weee = wee;
 assign countt = count;  
  
  // Idle State Detector
  
  idle_state_detector ISD ( .PV(PV),
                            .o(ISD_o)
                            );
                            
  // LFSR layer
  
  lfsr_16bit LFSR ( .enable(~ISD_o),
                    .reset(reset),
                    .clock(clock),
                    .q(In)
                   );
                   
  // First Tri-State Gate Layer
  
  tri_state_gate T0 ( .i(In[0]),
                      .enable(~ISD_o),
                      .o(i[0])
                    );
                    
  tri_state_gate T1 ( .i(In[1]),
                      .enable(~ISD_o),
                      .o(i[1])
                    );
                    
  tri_state_gate T2 ( .i(In[2]),
                      .enable(~ISD_o),
                      .o(i[2])
                    );
  
  tri_state_gate T3 ( .i(In[3]),
                      .enable(~ISD_o),
                      .o(i[3])
                    );
                    
  tri_state_gate T4 ( .i(In[4]),
                      .enable(~ISD_o),
                      .o(i[4])
                    );
                    
  tri_state_gate T5 ( .i(In[5]),
                      .enable(~ISD_o),
                      .o(i[5])
                    );   
                    
  tri_state_gate T6 ( .i(In[6]),
                      .enable(~ISD_o),
                      .o(i[6])
                    );
                    
  tri_state_gate T7 ( .i(In[7]),
                      .enable(~ISD_o),
                      .o(i[7])
                    );
                    
  tri_state_gate T8 ( .i(In[8]),
                      .enable(~ISD_o),
                      .o(i[8])
                    );     
                    
  tri_state_gate T9 ( .i(In[9]),
                      .enable(~ISD_o),
                      .o(i[9])
                     );
                    
  tri_state_gate T10 ( .i(In[10]),
                       .enable(~ISD_o),
                      .o(i[10])
                      );
                    
  tri_state_gate T11 ( .i(In[11]),
                       .enable(~ISD_o),
                       .o(i[11])
                     ); 
                    
  tri_state_gate T12 ( .i(In[12]),
                       .enable(~ISD_o),
                       .o(i[12])
                      );
                    
  tri_state_gate T13 ( .i(In[13]),
                       .enable(~ISD_o),
                       .o(i[13])
                      );
                    
  tri_state_gate T14 ( .i(In[14]),
                       .enable(~ISD_o),
                       .o(i[14])
                      ); 
                      
  tri_state_gate T15 ( .i(In[15]),
                       .enable(~ISD_o),
                       .o(i[15])
                      ); 
  
  // CUTs Layer
  
  ltv847 optocoupler ( .i(i[3:0]),
                       .o(Out[3:0])
                      );
  
  l298n_driver_1 driver1 ( .i(i[7:4]),
                           .o(Out[7:4])
                          );
                          
  l298n_driver_2 driver2 ( .i(i[11:8]),
                           .o(Out[11:8])
                          );
                          
  arduino_uno mcu ( .i(i[15:12]),
                    .o(Out[15:12])
                   );
                   
  // Second Tri-State Gate Layer
  
  tri_state_gate T16 ( .i(Out[0]),
                      .enable(~ISD_o),
                      .o(o[0])
                    );
                    
  tri_state_gate T17 ( .i(Out[1]),
                      .enable(~ISD_o),
                      .o(i[1])
                    );
                    
  tri_state_gate T18 ( .i(Out[2]),
                      .enable(~ISD_o),
                      .o(o[2])
                    );
  
  tri_state_gate T19 ( .i(Out[3]),
                      .enable(~ISD_o),
                      .o(o[3])
                    );
                    
  tri_state_gate T20 ( .i(Out[4]),
                      .enable(~ISD_o),
                      .o(o[4])
                    );
                    
  tri_state_gate T21 ( .i(Out[5]),
                      .enable(~ISD_o),
                      .o(o[5])
                    );   
                    
  tri_state_gate T22 ( .i(Out[6]),
                      .enable(~ISD_o),
                      .o(o[6])
                    );
                    
  tri_state_gate T23 ( .i(Out[7]),
                      .enable(~ISD_o),
                      .o(o[7])
                    );
                    
  tri_state_gate T24 ( .i(Out[8]),
                      .enable(~ISD_o),
                      .o(o[8])
                    );     
                    
  tri_state_gate T25 ( .i(Out[9]),
                      .enable(~ISD_o),
                      .o(o[9])
                     );
                    
  tri_state_gate T26 ( .i(Out[10]),
                       .enable(~ISD_o),
                      .o(o[10])
                      );
                    
  tri_state_gate T27 ( .i(Out[11]),
                       .enable(~ISD_o),
                       .o(o[11])
                     ); 
                    
  tri_state_gate T28 ( .i(Out[12]),
                       .enable(~ISD_o),
                       .o(o[12])
                      );
                    
  tri_state_gate T29 ( .i(Out[13]),
                       .enable(~ISD_o),
                       .o(o[13])
                      );
                    
  tri_state_gate T30 ( .i(Out[14]),
                       .enable(~ISD_o),
                       .o(o[14])
                      ); 
                      
  tri_state_gate T31 ( .i(Out[15]),
                       .enable(~ISD_o),
                       .o(o[15])
                      ); 
  
  // MISR Layer
  
  misr_8bit_first_half ( .d(o[7:0]),
                         .enable(~ISD_o),
                         .reset(reset),
                         .clock(clock),
                         .q(q[7:0])
                        );
                        
 misr_8bit_second_half ( .d(o[15:8]),
						 .y(w[7:0]),
                         .enable(~ISD_o),
                         .reset(reset),
                         .clock(clock),
                         .q(q[15:8])
                        );
                        
// BILBO (watchdog) Layer
watchdog_unit ( .x(Out[11:8]), 
				.y(w[7:0])
				);


// ROM Layer
// To be completed...
rom ( .addr(countt),
	  .clk(clock),
	  .din(q),
	  .dout(ROMOut),
	  .we(weee)
	  );
                
endmodule
  
                
                      
   
                    
                   
        
 
